/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 1999-2003 Tony Bybell
    Copyright (C) 2000-2003 Department of Computer Science
    The University of Manchester, Oxford Road, Manchester, M13 9PL, UK

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    `bsearch.c'
    Binary search compare predicates

    $Id: bsearch.c,v 1.21 2003/08/05 22:55:03 bardslea Exp $
*/

#include "analyzer.h"
#include "currenttime.h"
#include "utils.h"
#include "bsearch.h"
#include "strace.h"
#include "tracepane.h"

static TimeType max_compare_time_tc;
static TimeType *max_compare_pos_tc;

static int compar_timechain (const void *s1, const void *s2)
{
    TimeType key, obj, delta;
    TimeType *cpos;
    int rv;

    key = *((TimeType *) s1);
    obj = *(cpos = (TimeType *) s2);

    if ((obj <= key) && (obj > max_compare_time_tc))
    {
        max_compare_time_tc = obj;
        max_compare_pos_tc = cpos;
    }

    delta = key - obj;
    if (delta < 0)
        rv = -1;
    else if (delta > 0)
        rv = 1;
    else
        rv = 0;

    return (rv);
}

int bsearch_timechain (TimeType key)
{
    max_compare_time_tc = -2;
    max_compare_pos_tc = NULL;

    if (!timearray)
        return (-1);

    bsearch (&key, timearray, timearray_size, sizeof (TimeType), compar_timechain);
    if ((!max_compare_pos_tc) || (max_compare_time_tc < 0))
    {
        max_compare_pos_tc = timearray; /* aix bsearch fix */
    }

    return (max_compare_pos_tc - timearray);
}

int maxlen_trunc;
static char *maxlen_trunc_pos;
static char *trunc_asciibase;

static int compar_trunc (const void *s1, const void *s2)
{
    char *str;
    char vcache[2];
    int key, obj;

    str = (char *) s2;
    key = *((int *) s1);

    vcache[0] = *str;
    vcache[1] = *(str + 1);
    *str = '+';
    *(str + 1) = 0;
    obj = gdk_string_measure (wavefont, trunc_asciibase);
    *str = vcache[0];
    *(str + 1) = vcache[1];

    if ((obj <= key) && (obj > maxlen_trunc))
    {
        maxlen_trunc = obj;
        maxlen_trunc_pos = str;
    }

    return (key - obj);
}

char *bsearch_trunc (char *ascii, int maxlen)
{
    int len;

    if ((maxlen <= 0) || (!ascii) || (!(len = strlen (ascii))))
        return (NULL);

    maxlen_trunc = 0;
    maxlen_trunc_pos = NULL;

    bsearch (&maxlen, trunc_asciibase = ascii, len, sizeof (char), compar_trunc);
    return (maxlen_trunc_pos);
}

static int compar_symbols (const void *key, const void *v2)
{
    Symbol *s2;
    int rc;

    s2 = *((Symbol **) v2);
    rc = sigcmp ((char *) key, s2->name);
    return (rc);
}

Symbol *bsearch_symbols (char *ascii)
{
    Symbol **rc;

    if ((!ascii) || (!strlen (ascii)))
        return (NULL);
    rc = (Symbol **) bsearch (ascii, wave_symbols, wave_num_symbols, sizeof (Symbol *), compar_symbols);
    if (rc)
        return (*rc);
    else
        return (NULL);
}
