/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 1999-2003 Tony Bybell
    Copyright (C) 2000-2003 Department of Computer Science
    The University of Manchester, Oxford Road, Manchester, M13 9PL, UK

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    `color.h'
    Colour handling

    $Id: color.h,v 1.21 2003/11/09 22:21:33 bardslea Exp $
*/

#ifndef WAVE_COLOR_H
#define WAVE_COLOR_H

#include <stdlib.h>
#include <gtk/gtk.h>
#include "config/global.h"
#include "view.h"

/* wave_rgb_to_gdk_color
   wave_gdk_color_to_rgb : turn an {R, G, B} double triple into
	a GdkColor triple and back again */
extern void wave_rgb_to_gdk_color (const gdouble * src, GdkColor * dest);
extern void wave_gdk_color_to_rgb (const GdkColor * src, gdouble * dest);

/* wave_color_to_gdk_color
   wave_gdk_color_to_wave_color : turn into a GDK color from a single word
   	WaveColor and back again.  Note that this depends on the GdkColor being
   	populated (ie. not returned by gdk_gc_get_values) */
extern GdkColor wave_color_to_gdk_color (WaveColor color);
extern WaveColor wave_gdk_color_to_wave_color (GdkColor * color);

/* wave_color_to_gc : make a new graphics context with the
	wave colour `color' as the foreground colour */
extern GdkGC *wave_color_to_gc (GtkWidget * widget, WaveColor color);

/* wave_free_state_colors_gcs : free the GCs (well, unref) contained in the gcs
	field of the given TraceStateColors structure.  This is a prelude to
	deallocating the structure itself */
extern void wave_free_state_colors_gcs (TraceStateColors * state_colors);

/* wave_free_pane_colors_gcs : " " for pane colours */
extern void wave_free_pane_colors_gcs (TracePaneColors * pane_colors);

/* wave_get_state_color_gc : get the GC to draw a particular state in a
	particular trace.  Make the GC from the colour in the state_colors if
	it's not already there */
extern GdkGC *wave_get_state_color_gc (TraceStateColors * state_colors, TraceState state, WaveWindow *window);

/* wave_get_pane_color_gc : get the GC to draw a particular pane feature.
	Make the GC from the color in the pane_colors if it's not already there */
extern GdkGC *wave_get_pane_color_gc (TracePaneColors * pane_colors, TracePaneFeature feature, WaveWindow *window);

/* wave_gdk_color_is_light : returns true if the given color is too light to
	be easily seen on a white background */
extern gboolean wave_gdk_color_is_light (GdkColor * color);

#endif
