/*
    GTKWave Trace Configuration Library - libwaveconfig
    Copyright (C) 2002-2003 Esterel Technologies [http://www.esterel-technologies.com]
    Copyright (C) 2002-2003 The University of Manchester

    The authors of this library can be contacted through the GTKWave maintainer.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    `color.h'
    WaveColor and TraceDecor handling, keeping colour handling
    independent of Gdk

    $Id: color.h,v 1.19 2003/03/24 17:31:27 bardslea Exp $
*/

#ifndef WAVE_CONFIG_COLOR_H
#define WAVE_CONFIG_COLOR_H

#include <glib.h>

/* WaveColor is a 24 bit colour */
typedef struct
{
    unsigned char red, green, blue;
}
WaveColor;

/* TraceStateColors contains a list of colours for the
	trace states (and possibly a set of GCs.) */
typedef struct
{
    WaveColor *colors;          /* table of TraceState colours */
    void *gcs;                  /* Graphics Contexts, for use in the body
                                   of GTKWave */
}
TraceStateColors;

/* TracePaneColors contains a list of colours for the
	trace pane (i.e. global across traces in that pane) indexed
	by TracePaneFeature */
typedef struct
{
    WaveColor *colors;          /* Table of colours, indexed by TracePaneFeature */
    void *gcs;                  /* Graphics Contexts, for use in the body
                                   of GTKWave */
}
TracePaneColors;

/* A TraceDecor is a named set of state colours */
typedef struct
{
    char *name;
    TraceStateColors *state_colors;
}
TraceDecor;

/* string_to_wave_color : parse a colour value (#RRGGBB) to a new trace colour */
extern WaveColor string_to_wave_color (const char *str);

/* wave_color_to_string : produce an #RRGGBB string (newly allocated)
	for the given wave colour */
extern char *wave_color_to_string (WaveColor color);

/* new_trace_decor : allocate an empty decor */
extern TraceDecor *new_trace_decor (const char *name);

/* find_trace_decor_in_list : find a decor in a list of decors */
extern TraceDecor *find_trace_decor_in_list (GList * decors, const char *name);

/* {new,free}_trace_state_colors : create/destroy a state colour table with default entries */
extern TraceStateColors *new_trace_state_colors (void);
extern void free_trace_state_colors (TraceStateColors * state_colors);

/* copy_trace_state_colors : copy a state colours structure */
extern TraceStateColors *copy_trace_state_colors (TraceStateColors * state_colors);

/* new_trace_pane_colors : create a table of pane colours with default entries */
extern TracePaneColors *new_trace_pane_colors (void);

#endif
