/*
    GTKWave Trace Configuration Library - libwaveconfig
    Copyright (C) 2002-2003 Esterel Technologies [http://www.esterel-technologies.com]
    Copyright (C) 2002-2003 The University of Manchester

    The authors of this library can be contacted through the GTKWave maintainer.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    `global.c'
    WaveConfig, trace set global configuration

    $Id: global.c,v 1.22 2003/03/25 17:26:19 bardslea Exp $
*/

#include <glib.h>
#include "config/global.h"

ColorableFeatureInfo wave_trace_state_feature_info[] =
{
    {"Trace Font", "font", {0x00, 0xFF, 0x00}},
    {"Low bit", "low", {0x00, 0xFF, 0x00}},
    {"High bit", "high", {0x00, 0xFF, 0x00}},
    {"Unknown outline", "x", {0x00, 0xFF, 0x00}},
    {"Unknown fill", "xfill", {0x00, 0x80, 0x00}},
    {"Transition", "trans", {0x00, 0xFF, 0x00}},
    {"Hi-Z, Undriven bit", "mid", {0x00, 0xFF, 0x00}},
    {"Vector transition", "vtrans", {0x00, 0xFF, 0x00}},
    {"Vector outline", "vbox", {0x00, 0xFF, 0x00}},
    {"Unloaded fill", "unloaded", {0x80, 0x00, 0x00}},
    {"Analog", "analog", {0x00, 0xFF, 0x00}},
    {"Analog Clipped", "clip", {0xFF, 0x00, 0x00}},
	{"Handshake request", "req", {0xFF, 0x00, 0x00}},
	{"Handshake acknowledge", "ack", {0x00, 0x80, 0x00}},
	{"Handshake outline", "hbox", {0xff, 0xff, 0xff}},
	{NULL, NULL, {0,0,0}}
};
ColorableFeatureInfo wave_trace_pane_feature_info[] =
{
    {"Trace Pane Background", "back", {0x18, 0x18, 0x18}},
    {"Trace Pane Grid", "grid", {0x80, 0x80, 0x80}},
    {"Named Markers", "mark", {0x00, 0x00, 0xFF}},
    {"Primary Marker", "umark", {0xFF, 0xFF, 0x00}},
    {"Time/Marker Name Font", "pfont", {0xFF, 0xFF, 0xFF}},
	{NULL, NULL, {0,0,0}}
};

void (*config_log) (gboolean is_error, const char *fmt, ...) = NULL;

/* new_wave_config : make a new WaveConfig structure and set some
	elements to default values */
WaveConfig *new_wave_config (void)
{
    WaveConfig *ret = g_new (WaveConfig, 1);
    TraceDecor *default_decor = new_trace_decor ("default");
    TraceGroup *default_group = new_trace_group ("default");

    default_group->decor = default_decor;

    ret->decors = g_list_append (NULL, default_decor);
    ret->traces = NULL;
    ret->default_group = default_group;
    ret->default_decor = default_decor;
    ret->groups = g_list_append (NULL, default_group);

    ret->primary_marker = new_trace_marker ("primary", 0);
    ret->markers = g_list_append (NULL, ret->primary_marker);

    /* Set this to default to the smallest unit */
    ret->time_dimension = 'f';
    ret->time_scale = 1;

    ret->pane_colors = new_trace_pane_colors ();

    return ret;
}
