/*
    GTKWave Trace Configuration Library - libwaveconfig
    Copyright (C) 2002-2003 Esterel Technologies [http://www.esterel-technologies.com]
    Copyright (C) 2002-2003 The University of Manchester

    The authors of this library can be contacted through the GTKWave maintainer.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    `global.h'
    WaveConfig, trace set global configuration
    Top level header file for public consumption

    $Id: global.h,v 1.18 2003/03/24 17:31:27 bardslea Exp $
*/

#ifndef WAVE_CONFIG_GLOBAL_H
#define WAVE_CONFIG_GLOBAL_H

#include "config/color.h"
#include "config/trace.h"
#include "config/group.h"

/* Name, short name to default colour mapping for colourable features */
typedef struct 
{
	char *name;
	char *short_name;
	WaveColor default_color;
}
ColorableFeatureInfo;

/* Information on the state wise colourable features of each
	trace and the features shared between all the traces */
extern ColorableFeatureInfo wave_trace_state_feature_info[];
extern ColorableFeatureInfo wave_trace_pane_feature_info[];

/* Struct to save data from the XML configuration file */
typedef struct WaveConfig_
{
    GList *decors, *groups, *traces, *markers;
    TraceDecor *default_decor;
    TraceGroup *default_group;
    TracePaneColors *pane_colors;
    TraceMarker *primary_marker;

    /* Time resolution : time_dimension can be one of [smunpf],
       time_scale is 1, 10 or 100 indicating how many of the
       time_dimension units is a single time unit. eg.
       {'n', 10} indicates that the base time unit is 10ns.
       TimeType values are always stored in multiples of the
       time_dimension only, not including the time_scale */
    char time_dimension;
    unsigned int time_scale;
}
WaveConfig;

/* new_wave_config : make a new WaveConfig structure and set some
	elements to default values */
extern WaveConfig *new_wave_config (void);

/* Set config_log to allow the config library to report errors */
extern void (*config_log) (gboolean is_error, const char *fmt, ...);

/* parse_wave_config_file : parse a wave config file and populate the (pre-initialised
	using new_wave_config) structure `config' */
extern gboolean parse_wave_config_file (const char *filename, WaveConfig * config);

/* write_wave_config_file : write a wave config file from a WaveConfig, returns TRUE
	on success and moans (returning FALSE as well) on failure */
extern gboolean write_wave_config_file (const char *filename, WaveConfig * config, const char *banner);

#endif
