/*
    GTKWave Trace Configuration Library - libwaveconfig
    Copyright (C) 2002-2003 Esterel Technologies [http://www.esterel-technologies.com]
    Copyright (C) 2002-2003 The University of Manchester

    The authors of this library can be contacted through the GTKWave maintainer.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    `group.c'
    Trace group handling

    $Id: group.c,v 1.10 2003/02/05 12:03:08 bardslea Exp $
*/

#include <string.h>
#include "config/group.h"

/* new_trace_group : allocate a TraceGroup and set its name */
TraceGroup *new_trace_group (char *name)
{
    TraceGroup *group = g_new (TraceGroup, 1);

    group->state_colors = NULL;
    group->decor = NULL;
    group->name = g_strdup ((name ? name : ""));

    return group;
}

/* match_trace_group_name : function for g_list_find_custom to match a char * against a trace_group's name */
static gint match_trace_group_name (TraceGroup * trace_group, char *str)
{
    return strcmp ((trace_group->name ? trace_group->name : ""), str);
}

/* find_trace_group_in_list : find a group in a list of groups */
TraceGroup *find_trace_group_in_list (GList * groups, const char *name)
{
    GList *found = g_list_find_custom (groups, (char *) name, (GCompareFunc) match_trace_group_name);

    return (found ? (TraceGroup *) found->data : NULL);
}

/* free_trace_group : destructor for trace group structures */
void free_trace_group (TraceGroup * group)
{
    if (group->state_colors)
        free_trace_state_colors (group->state_colors);

    g_free (group->name);
    g_free (group);
}

/* copy_trace_group : deep copy a trace group */
TraceGroup *copy_trace_group (TraceGroup * group)
{
    TraceGroup *ret = new_trace_group (group->name);

    if (group->state_colors)
        ret->state_colors = copy_trace_state_colors (group->state_colors);
    ret->decor = group->decor;

    return ret;
}
