/*
    GTKWave Trace Configuration Library - libwaveconfig
    Copyright (C) 2002-2003 The University of Manchester
    Copyright (C) 1999-2003 Tony Bybell

    The authors of this library can be contacted through the GTKWave maintainer.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    `stime.h'
    Simulation Time Handling

    $Id: stime.h,v 1.7 2003/07/13 15:37:16 bardslea Exp $
*/

#ifndef WAVE_CONFIG_STIME_H
#define WAVE_CONFIG_STIME_H

#include <glib.h>

/* Useful place to discard debug info */
#define WAVE_DEBUG(arg)

/* Handle 64-bit oddities */
#ifdef G_HAVE_GINT64
#ifndef _MSC_VER
#define LL_LITERAL(x) x##LL
#define ULL_LITERAL(x) x##ULL
#define LL_PRINTF_FORMAT "%lld"
#define ULL_PRINTF_FORMAT "%llu"
#else
#define LL_LITERAL(x) x##i64
#define ULL_LITERAL(x) x##i64
#define LL_PRINTF_FORMAT "%I64d"
#define ULL_PRINTF_FORMAT "%I64u"
#endif
#else
#define LL_LITERAL(x) x
#define ULL_LITERAL(x) x
#define LL_PRINTF_FORMAT "%ld"
#define ULL_PRINTF_FORMAT "%lu"
#endif

/* Type used to represent simulation time, please please have 64b support */
#ifdef G_HAVE_GINT64
typedef gint64 TimeType;
typedef guint64 UTimeType;
#else
typedef long TimeType;
typedef unsigned long UTimeType;
#endif

/* Maximum length of formatted time = max length of 64b int + 3 (eg. " ns") + '\0' */
#define MAX_FORMATTED_TIME_LENGTH (24)
/* wave_format_time : format a TimeType into a string (using the
	given buffer) using the preferred dimension character, or a smaller
	one if necessary */
extern void wave_format_time (char *buf, TimeType time, char dimension);

/* wave_str_to_time_type : (hopefully!) 64b integer from string parser.
	Returns a TimeType value and sets *end_ptr (if not NULL) to the first
	non-numeric character at the end of the parsed number in the string.
	Returns 0 on failure. */
extern TimeType wave_str_to_time_type (const char *str, char **end_ptr);

/* wave_parse_time : parse a string with a time value and a dimension
	(ns/ms/s ...) into a TimeType value.  Dimension is the base dimension
	of the returned value.  If no dimension is specified in the string,
	assume that the units are `dimension' */
extern TimeType wave_parse_time (const char *buf, char dimension);

typedef struct
{
    char *name;
    UTimeType time;
}
TraceMarker;

/* {new,free}_trace_marker : make/destroy a marker, populate with copy of `name' and `time' */
extern TraceMarker *new_trace_marker (const char *name, UTimeType time);
extern void free_trace_marker (TraceMarker *marker);

/* find_trace_marker_in_list : find a marker in a list of markers */
extern TraceMarker *find_trace_marker_in_list (GList * markers, const char *name);

#endif
