/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 1999-2003 Tony Bybell
    Copyright (C) 2000-2003 Department of Computer Science
    The University of Manchester, Oxford Road, Manchester, M13 9PL, UK

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    `digital.c'
    Digtial drawing functions

    $Id: digital.c,v 1.13 2003/11/10 12:12:04 bardslea Exp $
*/

#include "digital.h"
#include "currenttime.h"
#include "value.h"
#include "rc.h"
#include "strace.h"
#include "help.h"
#include "log.h"
#include "tracepane.h"
#include "signalpane.h"

static void draw_roundends (WaveDrawingContext *context, int dir, int x, int y0, int y1, int yu)
{
    GdkPoint pnts[3];

    pnts[0].x = x;
    pnts[0].y = yu;
    pnts[1].y = y1;
    pnts[2].y = y0;

    if (!dir)
        pnts[1].x = pnts[2].x = x + 2;
    else
        pnts[1].x = pnts[2].x = x - 2;

	WAVE_DRAW_STATE_FILLED_POLYGON (context, TRACE_STATE_XFILL, pnts, 3);
}

/*
 * ----------------------------------------------------------------------
 *
 * Single Bit Traces
 *
 * ----------------------------------------------------------------------
 */

static void digital_bit_trace_start (WaveDrawingContext *context, Node *node, WaveEvent * ev, int y0, int y1)
{
    TraceState state = (ev->value.val == 'X' ? TRACE_STATE_X : TRACE_STATE_TRANS);

    WAVE_DRAW_STATE_LINE (context, state, 0, y0, 0, y1);
}

static void digital_bit_trace (WaveDrawingContext *context, Node *node, WaveEvent * ev, WaveEvent * ev2, int whichgc,
  gboolean first_at_this_time, int x0, int y0, int x1, int y1, int x, int ytext)
{
	TraceState state = (whichgc ? TRACE_STATE_X : TRACE_STATE_TRANS);
    char evval, ev2val;
    TimeType ev2tim;

    if (! ev)
        return;

    evval = wave_event_digital_value (context->trace->config, ev);
    if (! ev2)
    {                           /* ok, no next histentry, set value as X and time at tim.end */
		ev2val = evval;
        ev2tim = tims.end;       /* last time displayed */
    } else
    {
		ev2val = wave_event_digital_value (context->trace->config, ev2);
        ev2tim = ev2->time;
    }

    switch (evval)
    {
    case '0':
        WAVE_DRAW_STATE_LINE (context, TRACE_STATE_LOW, x0, y0, x1, y0);
        if (!x && ev2tim <= tims.end)
        {
            switch (ev2val)
            {
            case 'Z':
                WAVE_DRAW_STATE_LINE (context, state, x1, y0, x1, (y0 + y1) >> 1);
                break;
            case '1':
            case 'X':
                WAVE_DRAW_STATE_LINE (context, state, x1, y0, x1, y1);
                break;
            default:
            	break;
            }
        }
        break;

    case 'X':
#if 0
        if (ev2tim - (*h)->time <= 2)
        {
            WAVE_DRAW_STATE_LINE (context, TRACE_STATE_trans, x0, (ev2val == '0') ? y0 : y1, x1, (ev2val == '0') ? y1 : y0);
			/* Beep beep AB */
            WAVE_DRAW_STATE_LINE (context, TRACE_STATE_trans, x0, (ev2val == '0') ? y1 : y0, x1, (ev2val == '0') ? y0 : y1);
        } else
#endif
        {
            // gdk_draw_rectangle (gwdc->data.gdk.pixmap, TRACE_STATE_XFILL, TRUE, x0 + 1, y1, x1 - x0, y0 - y1 + 1);
            WAVE_DRAW_STATE_FILLED_RECTANGLE (context, TRACE_STATE_XFILL, x0 + 1, y1, x1, y0);
            WAVE_DRAW_STATE_LINE (context, TRACE_STATE_X, x0, y0, x1, y0);
            WAVE_DRAW_STATE_LINE (context, TRACE_STATE_X, x0, y1, x1, y1);

            if (!x && ev2tim <= tims.end)
            {
                WAVE_DRAW_STATE_LINE (context, state, x1, y0, x1, y1);
            }
        }
        break;

    case 'Z':
        {
            int yu = (y0 + y1) >> 1;

            WAVE_DRAW_STATE_LINE (context, TRACE_STATE_MID, x0, yu, x1, yu);
            if (!x && ev2tim <= tims.end)
            {
                switch (ev2val)
                {
                case '0':
                    WAVE_DRAW_STATE_LINE (context, state, x1, yu, x1, y0);
                    break;
                case '1':
                    WAVE_DRAW_STATE_LINE (context, state, x1, yu, x1, y1);
                    break;
                case 'X':
                    WAVE_DRAW_STATE_LINE (context, state, x1, y0, x1, y1);
                    break;
                default:
                    break;
                }
            }
            break;
        }

    case '1':
        WAVE_DRAW_STATE_LINE (context, TRACE_STATE_HIGH, x0, y1, x1, y1);
        if (!x && ev2tim <= tims.end)
        {
            switch (ev2val)
            {
            case '0':
                WAVE_DRAW_STATE_LINE (context, state, x1, y1, x1, y0);
                break;
            case 'Z':
                WAVE_DRAW_STATE_LINE (context, state, x1, y1, x1, (y0 + y1) >> 1);
                break;
            case 'X':
                WAVE_DRAW_STATE_LINE (context, state, x1, y0, x1, y1);
                break;
            default:
                break;
            }
        }
        /* fallthrough on break */
    default:
        break;
    }
    if (x && x1 < wave_window->traces.scale.wave_pane_width)
    {
        int width = x - x1;

        // gdk_draw_rectangle (gwdc->data.gdk.pixmap, TRACE_STATE_XFILL, TRUE, x1 + 1, y1, x - x1, y0 - y1 + 1);
        WAVE_DRAW_STATE_FILLED_RECTANGLE (context, TRACE_STATE_XFILL, x1 + 1, y1, x, y0);
        WAVE_DRAW_STATE_LINE (context, TRACE_STATE_UNLOADED, x1, y0, x1, y1);
        WAVE_DRAW_STATE_LINE (context, TRACE_STATE_UNLOADED, x1, y0, x, y0);
        WAVE_DRAW_STATE_LINE (context, TRACE_STATE_UNLOADED, x1, y1, x, y1);
        if (width > wave_flags.vector_padding)
        {
            char ascii[2];

            ascii[0] = '?';
            ascii[1] = '\0';
            wave_draw_truncated_string (context, x1 + 2, ytext, x, width, ascii);
        }
    }
}

static void digital_bit_spike (WaveDrawingContext *context, Node *node, int x0, int y0, int x1, int y1)
{
    WAVE_DRAW_STATE_LINE (context, TRACE_STATE_TRANS, x1, y0, x1, y1);
}

/*
 * ----------------------------------------------------------------------
 *
 * Single Value Multi-Bit Traces
 *
 * ----------------------------------------------------------------------
 */

static void digital_value_trace1 (WaveDrawingContext *context, Node *node, int *trans, int type, int lasttype, int x0, int y0, int y1, int yu)
{
    TraceState last_state = (lasttype == 'X' ? TRACE_STATE_X : TRACE_STATE_VTRANS);
    TraceState state = (type == 'X' ? TRACE_STATE_X : TRACE_STATE_VTRANS);

    if (wave_flags.roundcap_vectors_flag)
    {
        if (*trans)
        {
            /* FIXME, trans? */
			draw_roundends (context, 1, x0, y0, y1, yu);
            WAVE_DRAW_STATE_LINE (context, TRACE_STATE_UNLOADED, x0 - 2, y0, x0, yu);
            WAVE_DRAW_STATE_LINE (context, TRACE_STATE_UNLOADED, x0, yu, x0 - 2, y1);
            WAVE_DRAW_STATE_LINE (context, TRACE_STATE_VTRANS, x0, yu, x0 + 2, y0);
            WAVE_DRAW_STATE_LINE (context, TRACE_STATE_VTRANS, x0, yu, x0 + 2, y1);
        } else
        {
            /* FIXME, is this the right patch? */
            /* was:
             * gdk_draw_line(gwdc->data.gdk.pixmap, gc_vtrans, x0 - 2, y0, x0 + 2, y1);
             * gdk_draw_line(gwdc->data.gdk.pixmap, gc_vtrans, x0 + 2, y0, x0 - 2, y1);
             */
            if (type == 'Z')
            {
                if (lasttype != -1)
                {
                    WAVE_DRAW_STATE_LINE (context, last_state, x0 - 1, y0, x0, yu);
                    WAVE_DRAW_STATE_LINE (context, last_state, x0, yu, x0 - 1, y1);
                }
            } else if (lasttype == 'Z')
            {
                WAVE_DRAW_STATE_LINE (context, state, x0 + 1, y0, x0, yu);
                WAVE_DRAW_STATE_LINE (context, state, x0, yu, x0 + 1, y1);
            } else if (lasttype != type)
            {
                WAVE_DRAW_STATE_LINE (context, last_state, x0 - 1, y0, x0, yu);
                WAVE_DRAW_STATE_LINE (context, last_state, x0, yu, x0 - 1, y1);
                WAVE_DRAW_STATE_LINE (context, state, x0 + 1, y0, x0, yu);
                WAVE_DRAW_STATE_LINE (context, state, x0, yu, x0 + 1, y1);
            } else
            {
                WAVE_DRAW_STATE_LINE (context, state, x0 - 2, y0, x0 + 2, y1);
                WAVE_DRAW_STATE_LINE (context, state, x0 + 2, y0, x0 - 2, y1);
            }
        }
        *trans = 0;
    } else
    {
        WAVE_DRAW_STATE_LINE (context, state, x0, y0, x0, y1);
    }
}

static void digital_value_trace2 (WaveDrawingContext *context, Node *node, WaveEvent * ev, int *trans, int type,
	int x, int x0, int y0, int x1, int y1, int yu, int ytext)
{
    if (type == 'Z')
    {
        if (wave_flags.roundcap_vectors_flag)
            WAVE_DRAW_STATE_LINE (context, TRACE_STATE_MID, x0 + 1, yu, x1 - 1, yu);
        else WAVE_DRAW_STATE_LINE (context, TRACE_STATE_MID, x0, yu, x1, yu);
    } else
    {
		TraceState state;
        int width;

        if (type == '0')
            state = TRACE_STATE_VBOX;
        else state = TRACE_STATE_X;

        if (wave_flags.roundcap_vectors_flag)
        {
            WAVE_DRAW_STATE_LINE (context, state, x0 + 2, y0, x1 - 2, y0);
            WAVE_DRAW_STATE_LINE (context, state, x0 + 2, y1, x1 - 2, y1);
        } else
        {
            WAVE_DRAW_STATE_LINE (context, state, x0, y0, x1, y0);
            WAVE_DRAW_STATE_LINE (context, state, x0, y1, x1, y1);
        }

        /* FIXME, correct?
         * if (wave_flags.roundcap_vectors_flag) {
         *     gdk_draw_line(gwdc->data.gdk.pixmap, TRACE_STATE_vbox, x0 + 2, y0, x1 - 2, y0);
         *     gdk_draw_line(gwdc->data.gdk.pixmap, TRACE_STATE_vbox, x0 + 2, y1, x1 - 2, y1);
         * } else {
         *     gdk_draw_line(gwdc->data.gdk.pixmap, TRACE_STATE_vbox, x0, y0, x1, y0);
         *     gdk_draw_line(gwdc->data.gdk.pixmap, TRACE_STATE_vbox, x0, y1, x1, y1);
         * }
         */
        if (x0 < 0)
            x0 = 0;             /* fixup left margin */
        if ((width = x1 - x0) > wave_flags.vector_padding)
        {
            char *ascii = wave_event_value_string (context->trace, ev);

            wave_draw_truncated_string (context, x0 + 2, ytext, x1, width, ascii);
            g_free (ascii);
        }
        if (x && x1 < wave_window->traces.scale.wave_pane_width)
        {
            if (wave_flags.roundcap_vectors_flag)
            {
                draw_roundends (context, 0, x1, y0, y1, yu);
                // gdk_draw_rectangle (gwdc->data.gdk.pixmap, TRACE_STATE_XFILL, TRUE, x1 + 2, y1, x - x1 - 4, y0 - y1 + 1);
                WAVE_DRAW_STATE_FILLED_RECTANGLE (context, TRACE_STATE_XFILL, x1 + 2, y1, x - 2, y0);

                WAVE_DRAW_STATE_LINE (context, TRACE_STATE_VTRANS, x1 - 2, y0, x1, yu);
                WAVE_DRAW_STATE_LINE (context, TRACE_STATE_VTRANS, x1 - 2, y1, x1, yu);
                WAVE_DRAW_STATE_LINE (context, TRACE_STATE_UNLOADED, x1, yu, x1 + 2, y0);
                WAVE_DRAW_STATE_LINE (context, TRACE_STATE_UNLOADED, x1, yu, x1 + 2, y1);

                WAVE_DRAW_STATE_LINE (context, TRACE_STATE_UNLOADED, x1 + 2, y0, x - 2, y0);
                WAVE_DRAW_STATE_LINE (context, TRACE_STATE_UNLOADED, x1 + 2, y1, x - 2, y1);
            } else
            {
                WAVE_DRAW_STATE_LINE (context, TRACE_STATE_VTRANS, x1, y0, x1, y1);
                // gdk_draw_rectangle (gwdc->data.gdk.pixmap, TRACE_STATE_XFILL, TRUE, x1 + 1, y1, x - x1, y0 - y1 + 1);
                WAVE_DRAW_STATE_FILLED_RECTANGLE (context, TRACE_STATE_XFILL, x1 + 1, y1, x, y0);
                WAVE_DRAW_STATE_LINE (context, TRACE_STATE_UNLOADED, x1, y0, x, y0);
                WAVE_DRAW_STATE_LINE (context, TRACE_STATE_UNLOADED, x1, y1, x, y1);
            }
            if ((width = x - x1) > wave_flags.vector_padding)
            {
                char ascii[2];

                ascii[0] = '?';
                ascii[1] = '\0';
                wave_draw_truncated_string (context, x1 + 2, ytext, x, width, ascii);
            }
            *trans = 1;
        }
    }
}

static gboolean digital_compat (TraceEnt * t)
{
    /*
     * The default is *always* compatible.
     */
    return TRUE;
}

TraceDrawer digital_trace_drawer = {
    "digital", digital_compat,

    digital_bit_trace_start,
    digital_bit_trace, digital_bit_spike,

    digital_value_trace1,
    digital_value_trace2,

    NULL,
};

void on_edit_display_format_digital_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    TraceEnt *t;

    if (wave_help ("Edit->Display Format->Digital", "formats the first selected as a normal digital trace"))
        return;

    /* don't mess with sigs when strace or wave_dnd active */
	if (straces)
	{
		strace_error ();
		return;
	}
	if (wave_dnd_state)
	{
		wave_dnd_error ();
		return;
	}

    WAVE_DEBUG (wave_log (FALSE, "WAVE_DEBUG: Display Format = Digital\n"));

    for (t = traces.first; t; t = t->next)
    {
        if (TRACE_IS_SELECTED (t->config))
        {
            unset_trace_drawer (t);
            wave_redraw (wave_window);
            return;
        }
    }
    wave_log (TRUE, "Please select a trace to modify first.\n");
}

