/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 1999-2003 Tony Bybell
    Copyright (C) 2000-2003 Department of Computer Science
    The University of Manchester, Oxford Road, Manchester, M13 9PL, UK

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    `fonts.c'
    Font definitions

    $Id: fonts.c,v 1.21 2003/11/10 16:35:49 bardslea Exp $
*/

#include "currenttime.h"
#include "fonts.h"
#include "rc.h"
#include "tracepane.h"
#include "signalpane.h"

char *fontname_signals = NULL;
char *fontname_waves = NULL;
int font_height_override = 0;

#ifdef _MSC_VER
#define NAME_BUFFER_LEN (32)
#  include <windows.h>
/*
 * This code has been found at http://www.ecn.wfu.edu/~cottrell/gtk_win32/
 * (C) Allin Cottrell, Wake Forest University
 * cottrell@wfu.edu 
 * GPL license ok.
 */
char *get_default_font (void)
{
    HDC h_dc;
    HGDIOBJ h_font;
    TEXTMETRIC tm;
    char name[NAME_BUFFER_LEN];
    int len, pix_height;
    char *font_name;

    h_dc = CreateDC ("DISPLAY", NULL, NULL, NULL);
    if (h_dc == NULL)
        return NULL;
    h_font = GetStockObject (DEFAULT_GUI_FONT);
    if (h_font == NULL || !SelectObject (h_dc, h_font))
    {
        DeleteDC (h_dc);
        return NULL;
    }
    len = GetTextFace (h_dc, NAME_BUFFER_LEN, name);
    if (len <= 0)
    {
        DeleteDC (h_dc);
        return NULL;
    }
    if (!GetTextMetrics (h_dc, &tm))
    {
        DeleteDC (h_dc);
        return NULL;
    }
    pix_height = tm.tmHeight;
    DeleteDC (h_dc);
    font_name = g_strdup_printf ("-*-%s-*-*-*-*-%i-*-*-*-p-*-iso8859-1", name, pix_height);

    return font_name;
}
#else
char *get_default_font (void)
{
    return g_strdup (WAVE_FONT_WAVE_SMALL);
}
#endif

void wave_load_all_fonts (WaveWindow *window)
{
    if ((fontname_signals) && (strlen (fontname_signals)))
    {
        signalfont = gdk_font_load (fontname_signals);
    }

    if (!signalfont)
    {
        signalfont = window->window->style->font;
    }

    fontheight = (signalfont->ascent + signalfont->descent) + 4;
    if (font_height_override > fontheight)
    {
        fontheight = font_height_override;
    }

    if ((fontname_waves) && (strlen (fontname_waves)))
    {
        wavefont = wavefont_smaller = gdk_font_load (fontname_waves);
    } else
    {
        char *font_name = get_default_font ();

        wavefont_smaller = gdk_font_load (font_name);
        wavefont = wavefont_smaller;
        g_free (font_name);
    }

    if (!wavefont)
    {
        wavefont = wavefont_smaller = window->window->style->font;
    }

    if (signalfont->ascent < wavefont->ascent)
    {
        fprintf (stderr, "Signalfont is smaller (%d pt) than wavefont (%d pt).\n",
          signalfont->ascent, wavefont->ascent);
        fprintf (stderr, "Using wavefont for both signals and waves.\n");
        signalfont = wavefont;
    }

    if (signalfont->ascent > 100)
    {
        fprintf (stderr, "Fonts are too big!  Try fonts with a smaller size.  Exiting!\n");
        exit (1);
    }

    window->traces.scale.time_line_y = wavefont->ascent + 1;
}
