/*
	The GTKWave electronic waveform viewer
	Copyright (C) 2000-2003 Department of Computer Science
	The University of Manchester, Oxford Road, Manchester, M13 9PL, UK
	Copyright (C) 1999-2003 Tony Bybell

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

	`gui_cb.c'
	GUI Callbacks, this file is tending towards just containing main window
	menu/button callbacks with the dialogue box callbacks in separate files.

	$Id: gui_cb.c,v 1.101 2003/11/13 11:05:37 bardslea Exp $
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <stdarg.h>
#include <errno.h>

#include "gui_cb.h"
#include "gui_if.h"
#include "support.h"

#include "currenttime.h"
#include "strace.h"
#include "utils.h"
#include "log.h"
#include "main.h"
#include "print.h"
#include "reader.h"
#include "draw.h"
#include "color.h"
#include "tracefile.h"
#include "polling.h"
#include "tracepane.h"
#include "signalpane.h"
#include "str.h"

GtkWidget *build_main_window (void)
{
    wave_window = wave_new_window ();

    return wave_window->window;
}

#ifndef _MSC_VER
static void menu_new_viewer_cleanup (GtkWidget * widget, gpointer data)
{
    pid_t pid;
    char *fileselbox_text;

    if (TRUE) /* filesel_ok */
    {
    	char *program_name = g_get_prgname ();
        /*
         * for some reason, X won't let us double-fork in order to
         * cleanup zombies.. *shrug*
         */
        pid = fork ();
        if (((int) pid) < 0)
        {
            wave_log (TRUE, "ERROR: fork() failed: %s\n", g_strerror (errno));
            return;             /* not much we can do about this.. */
        }

        if (pid)
        {                       /* parent==original server_pid */
            return;
        }

        execlp (program_name, program_name, *fileselbox_text, NULL);
        fprintf (stderr, "ERROR: execlp() failed: %s\n", g_strerror (errno));
        exit (0);               /* control never gets here if successful */
    }
}
#endif /* _MSC_VER */

void on_file_open_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Open Wave", "lets you open a new trace window on a different trace file"))
        return;

#ifdef _MSC_VER
    wave_log (TRUE, "That function is not supported on Windows yet.\n");
#else
/* FUCK this */
    menu_new_viewer_cleanup (NULL, NULL);
#endif /* _MSC_VER */
}

void on_file_quit_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Quit", "makes the program exit"))
        return;

    gtk_main_quit ();
}

#define DO_EDIT_LOCK \
	if (straces) {strace_error();return;} \
	if (wave_dnd_state) {wave_dnd_error();return;}

void on_edit_extend_trace_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    TraceEnt *t;
    gboolean some_traces_changed = FALSE;

    if (wave_help ("Extend Trace Vertically", "No help"))
        return;

    DO_EDIT_LOCK;               /* don't mess with sigs when strace or wave_dnd active */

    for (t = traces.first; t; t = t->next)
    {
        if (TRACE_IS_SELECTED (t->config) && ! TRACE_IS_EXTENSION (t->config))
        {
			TraceEnt *extension_trace;

			some_traces_changed = TRUE;
            t->config->height++;
            /* Something has been highlighted, go ahead */
            /* FIXME, this really has to do the insertion at the right point
               and work for all selected traces */
            extension_trace = wave_insert_separator_trace (NULL, NULL, t);
            trace_config_set_flags (extension_trace->config, TRACE_FLAGS_EXTENSION | TRACE_FLAGS_SELECTED, TRUE);

        }
    }

	if (some_traces_changed)
	{
		wave_max_signal_length (wave_window);
        wave_redraw (wave_window);
	} else
		wave_log (TRUE, "Please select traces to extend first.\n");
}

void on_edit_contract_trace_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    TraceEnt *t;
    gboolean some_traces_changed = FALSE;

    if (wave_help ("Contract Trace Vertically", "No help"))
        return;

    DO_EDIT_LOCK;               /* don't mess with sigs when strace or wave_dnd active */

    for (t = traces.first; t; t = t->next)
    {
        if (TRACE_IS_SELECTED (t->config) && ! TRACE_IS_EXTENSION (t->config))
        {
			some_traces_changed = TRUE;

			if (t->config->height > 1)
			{
				t->config->height--;

				remove_trace (t->next, TRUE);
            }
        }
    }

	if (some_traces_changed)
	{
		wave_max_signal_length (wave_window);
        wave_redraw (wave_window);
	} else
		wave_log (TRUE, "Please select traces to contract first.\n");
}

void on_edit_insert_separator_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Insert Blank", "inserts a blank trace after the last selected trace. If no traces are selected, the blank is inserted after the last trace. Note that this function is disabled when Pattern Search is active"))
        return;

    DO_EDIT_LOCK;               /* don't mess with sigs when strace or wave_dnd active */

    WAVE_DEBUG (wave_log (FALSE, "WAVE_DEBUG: Insert Blank Trace\n"));

    wave_insert_separator_trace (NULL, NULL, NULL);
    wave_redraw (wave_window);
}

void on_edit_insert_comment_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Insert Comment", "inserts a comment trace after the last selected trace. If no traces are selected, the comment is inserted after the last trace. Note that this function is disabled when Pattern Search is active"))
        return;

    DO_EDIT_LOCK;               /* don't mess with sigs when strace or wave_dnd active */

    WAVE_DEBUG (wave_log (FALSE, "WAVE_DEBUG: Insert Comment Trace\n"));

	gtk_widget_show (create_comment_dialog ());
}

void on_comment_dialog_ok_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *entry = lookup_widget (GTK_WIDGET (button), "comment_entry");
	char *comment = gtk_entry_get_text (GTK_ENTRY (entry));

    wave_insert_separator_trace (comment, NULL, NULL);
    wave_window->signals.window_width_dirty = TRUE;
    wave_max_signal_length (wave_window);
	wave_redraw (wave_window);

	on_comment_dialog_cancel_clicked (button, user_data);
}

void on_comment_dialog_cancel_clicked (GtkButton *button, gpointer user_data)
{
	on_comment_dialog_delete_event (widget_top_parent (GTK_WIDGET (button)), NULL, NULL);
}

gboolean on_comment_dialog_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	gtk_widget_destroy (widget);
	return TRUE;
}

void on_about_dialog_close_clicked (GtkButton *button, gpointer user_data)
{
	on_about_dialog_delete_event (widget_top_parent (GTK_WIDGET (button)), NULL, NULL);
}

gboolean on_about_dialog_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	gtk_widget_destroy (widget);
	return TRUE;
}

void on_edit_change_trace_name_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Change Trace Name", "only works when at least one trace has been selected. With this function, you will be prompted for an alias name for the first selected trace. After successfully aliasing a trace, the aliased trace will be unselected. Single bits will be marked with a leading \"+\" and vectors will have no such designation. The purpose of this is to provide a fast method of determining which trace names are real and which ones are aliases. Note that this function is disabled when Pattern Search is active"))
        return;

    DO_EDIT_LOCK;               /* don't mess with sigs when strace or wave_dnd active */

/* FIXME, check to see if any traces are selected
    target = NULL;
    for (t = traces.first; t; t = t->next)
    {
        if (TRACE_IS_SELECTED (t->config))
        {
            target = t;
            break;
        }
    }
*/

	gtk_widget_show (create_trace_name_dialog ());
}

void on_trace_name_dialog_ok_clicked (GtkButton *button, gpointer user_data)
{
    TraceEnt *t, *target;
	GtkWidget *entry = lookup_widget (GTK_WIDGET (button), "trace_name_entry");
	char *trace_name = gtk_entry_get_text (GTK_ENTRY (entry));

	/* Change name of first selected trace */
    target = NULL;
    for (t = traces.first; t; t = t->next)
    {
        if (TRACE_IS_SELECTED (t->config))
        {
            target = t;
            break;
        }
    }

    if (target)
    {
		if (TRACE_IS_ALIAS (target->config))
			g_free (target->config->name);

		trace_config_set_flags (target->config, TRACE_FLAGS_ALIAS, TRUE);

		if (! TRACE_IS_SEPARATOR (target->config))
		{
			char *name = g_new (char, strlen (trace_name) + 3);

			strcpy (name, "+ ");
			strcpy (name + 2, trace_name);

			wave_substitute_char (name + 2, ' ', '_');

			target->config->name = name;
		} else {
			target->config->name = g_strdup (trace_name);
		}

		trace_config_set_flags (target->config, TRACE_FLAGS_SELECTED, FALSE);

		wave_window->signals.window_width_dirty = TRUE;
		wave_max_signal_length (wave_window);
		wave_redraw (wave_window);
    } else
    {
        wave_log (TRUE, "Must select a trace to change its name it!\n");
    }

	on_trace_name_dialog_cancel_clicked (button, user_data);
}

void on_trace_name_dialog_cancel_clicked (GtkButton *button, gpointer user_data)
{
	on_trace_name_dialog_delete_event (widget_top_parent (GTK_WIDGET (button)), NULL, NULL);
}

gboolean on_trace_name_dialog_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	gtk_widget_destroy (widget);
	return TRUE;
}

void on_edit_cut_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Cut", "removes selected signals from the display and places them in an offscreen cut buffer for later Paste operations. Cut implicitly destroys the previous contents of the cut buffer. Note that this function is disabled when Pattern Search is active"))
        return;

    DO_EDIT_LOCK;               /* don't mess with sigs when strace or wave_dnd active */

    WAVE_DEBUG (wave_log (FALSE, "WAVE_DEBUG: Cut Traces\n"));

    if (cut_buffer ())
    {
        wave_max_signal_length (wave_window);
		wave_redraw (wave_window);
    } else
    {
        wave_log (TRUE, "Must select at least one signal to cut.\n");
    }
}

void on_edit_paste_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Paste", "pastes signals from an offscreen cut buffer and places them in a group after the last selected signal, or at the end of the display if no signal is selected. Paste implicitly destroys the previous contents of the cut buffer. Note that this function is disabled when Pattern Search is active"))
        return;

    DO_EDIT_LOCK;               /* don't mess with sigs when strace or wave_dnd active */

    WAVE_DEBUG (wave_log (FALSE, "WAVE_DEBUG: Paste Traces\n"));

    if (paste_buffer (NULL))
    {
        wave_max_signal_length (wave_window);
		wave_redraw (wave_window);
		wave_signal_pane_adjust_after_insertion (wave_window);
    }
}

static void set_trace_data_format (TraceFormat format)
{
    TraceEnt *t;
    int fix = 0;

    for (t = traces.first; t; t = t->next)
    {
        if (TRACE_IS_SELECTED (t->config))
        {
            t->config->format = format;
            fix = 1;
        }
    }
    if (fix)
    {
        wave_window->signals.window_width_dirty = TRUE;
        wave_max_signal_length (wave_window);
		wave_redraw (wave_window);
    }
}
static void toggle_trace_flag (TraceFlags flag)
{
    TraceEnt *t;
    int fix = 0;

    for (t = traces.first; t; t = t->next)
    {
        if (TRACE_IS_SELECTED (t->config))
        {
            trace_config_toggle_flags (t->config, flag);
            fix = 1;
        }
    }
    if (fix)
    {
        wave_window->signals.window_width_dirty = TRUE;
        wave_max_signal_length (wave_window);
		wave_redraw (wave_window);
    }
}

void on_edit_data_format_hex_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	if (wave_help ("Data Format->Hex", "steps through all selected traces and ensure that vectors with this qualifier will be displayed with hexadecimal values"))
        return;
    set_trace_data_format (TRACE_FORMAT_HEX);
}

void on_edit_data_format_decimal_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	if (wave_help ("Data Format->Decimal", "steps through all selected traces and ensure that vectors with this qualifier will be displayed with decimal values"))
        return;
    set_trace_data_format (TRACE_FORMAT_DEC);
}

void on_edit_data_format_signed_decimal_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	if (wave_help ("Data Format->Signed", "steps through all selected traces and ensure that vectors with this qualifier will be displayed with sign-extended decimal values"))
        return;
    set_trace_data_format (TRACE_FORMAT_SIGNED_DEC);
}

void on_edit_data_format_octal_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	if (wave_help ("Data Format->Oct", "steps through all selected traces and ensure that vectors with this qualifier will be displayed with octal values"))
        return;
    set_trace_data_format (TRACE_FORMAT_OCT);
}

void on_edit_data_format_binary_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	if (wave_help ("Data Format->Bin", "steps through all selected traces and ensure that vectors with this qualifier will be displayed with binary values"))
        return;
    set_trace_data_format (TRACE_FORMAT_BIN);
}

void on_edit_data_format_ascii_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	if (wave_help ("Data Format->ASCII", "steps through all selected traces and ensure that vectors with this qualifier will be displayed with ASCII values"))
        return;
    set_trace_data_format (TRACE_FORMAT_ASCII);
}

void on_edit_data_format_justify_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	if (wave_help ("Data Format->Justify", "steps through all selected traces and toggles them between being left and right justified"))
        return;
    toggle_trace_flag (TRACE_FLAGS_RJUSTIFIED);
}

void on_edit_data_format_invert_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	if (wave_help ("Data Format->Invert", "steps through all selected traces and inverts all bits in the value"))
        return;
    toggle_trace_flag (TRACE_FLAGS_INVERTED);
}

void on_edit_data_format_reverse_bits_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	if (wave_help ("Data Format->Reverse", "steps through all selected traces and reverses the order of all bits in the value"))
        return;
    toggle_trace_flag (TRACE_FLAGS_REVERSED);
}

/* ---------------------------------------------------------------------- */

void on_edit_hide_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    TraceEnt *t;
    int fix = 0;

    if (wave_help ("Exclude", "blanks out the waveform for all selected traces"))
        return;

    for (t = traces.first; t; t = t->next)
    {
        if (TRACE_IS_SELECTED (t->config))
        {
            trace_config_set_flags (t->config, TRACE_FLAGS_HIDDEN, TRUE);
            fix = 1;
        }
    }
    if (fix)
    {
        wave_window->signals.window_width_dirty = TRUE;
        wave_max_signal_length (wave_window);
		wave_redraw (wave_window);
    }
}

void on_edit_show_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    TraceEnt *t;
    int fix = 0;

    if (wave_help ("Show", "reveals the waveform for all selected traces"))
        return;

    for (t = traces.first; t; t = t->next)
    {
        if (TRACE_IS_SELECTED (t->config))
        {
            trace_config_set_flags (t->config, TRACE_FLAGS_HIDDEN, FALSE);
            fix = 1;
        }
    }
    if (fix)
    {
        wave_window->signals.window_width_dirty = TRUE;
        wave_max_signal_length (wave_window);
		wave_redraw (wave_window);
    }
}

static void regexp_select_generic (int mode, char *re)
{
    TraceEnt *t;
    char dirty = 0;

    wave_regex_compile (re);

    for (t = traces.first; t; t = t->next)
    {
        char *pnt = t->config->name ? t->config->name : ""; /* handle (really) blank lines */

        if (*pnt == '+')
        {                       /* skip alias prefix if present */
            while (*(++pnt) == ' ');
        }

        if (wave_regex_match (pnt))
        {
            if (!mode)
                trace_config_set_flags (t->config, TRACE_FLAGS_SELECTED, FALSE);
            dirty = 1;
        }
    }

    if (dirty)
		wave_redraw (wave_window);
}

void on_edit_select_regexp_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Select Regexp", "brings up a dialog box that asks for a regular expression that may contain text with POSIX regular expressions. All traces matching this RE will be selected"))
        return;

	gtk_widget_show (create_select_regexp_dialog ());
}

void on_select_regexp_dialog_ok_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *entry = lookup_widget (GTK_WIDGET (button), "select_egexp_entry");
	char *regexp = gtk_entry_get_text (GTK_ENTRY (entry));

	regexp_select_generic (1, regexp);

	on_select_regexp_dialog_cancel_clicked (button, user_data);
}

void on_select_regexp_dialog_cancel_clicked (GtkButton *button, gpointer user_data)
{
	on_select_regexp_dialog_delete_event (widget_top_parent (GTK_WIDGET (button)), NULL, NULL);
}

gboolean on_select_regexp_dialog_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	gtk_widget_destroy (widget);
	return TRUE;
}

void on_edit_select_all_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    TraceEnt *t;

    if (wave_help ("Select All", "selects all displayed traces"))
        return;

    if (wave_dnd_state)
    {
        wave_dnd_error ();
        return;
    }
    /* don't mess with sigs when wave_dnd active */
    if (traces.first)
    {
        for (t = traces.first; t; t = t->next)
            trace_config_set_flags (t->config, TRACE_FLAGS_SELECTED, TRUE);
		wave_redraw (wave_window);
    }
}

void on_edit_unselect_all_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    TraceEnt *t;

    if (wave_help ("Unselect All", "unselects all displayed traces"))
        return;

    if (wave_dnd_state)
    {
        wave_dnd_error ();
        return;
    }
    /* don't mess with sigs when wave_dnd active */
    if (traces.first)
    {
        for (t = traces.first; t; t = t->next)
            trace_config_set_flags (t->config, TRACE_FLAGS_SELECTED, FALSE);
		wave_redraw (wave_window);
    }
}

void on_traces_menu_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    /* Does nothing for now */
}

void on_traces_sort_ascii_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Sort->ASCII Order", "arranged the traces in ASCII order, with blank traces sorted to the bottom"))
        return;

    if (wave_dnd_state)
    {                           /* don't mess with sigs when wave_dnd active */
        wave_dnd_error ();
        return;
    }

    if (traces.first && alphebetise_traces (1))
		wave_redraw (wave_window);
}

void on_traces_sort_dict_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Sort->Dictionary Order", "arranged the traces in ASCII order ignoring case, with blank traces sorted to the bottom"))
        return;

    if (wave_dnd_state)
    {                           /* don't mess with sigs when wave_dnd active */
        wave_dnd_error ();
        return;
    }

    if (traces.first && alphebetise_traces (0))
		wave_redraw (wave_window);
}

void on_traces_sort_sigsort_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Sort->Signal Order", "arranged the traces in order taking the numeric portions into account, with blank traces sorted to the bottom"))
        return;

    if (wave_dnd_state)
    {                           /* don't mess with sigs when wave_dnd active */
        wave_dnd_error ();
        return;
    }

    if (traces.first && alphebetise_traces (2))
		wave_redraw (wave_window);
}

void on_traces_sort_rev_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Sort->Reverse Order", "reverses the order of all signals"))
        return;

    if (wave_dnd_state)
    {                           /* don't mess with sigs when wave_dnd active */
        wave_dnd_error ();
        return;
    }

    if (traces.first && reverse_traces ())
		wave_redraw (wave_window);
}

static void menu_tracesearchbox_callback (GtkWidget * widget, gpointer data)
{
    /* EMPTY */
}

void on_traces_pattern_search_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    TraceEnt *t;

    if (wave_help ("Pattern Search", "only works when at least one trace is selected. A requester will appear that lists all the selected traces (maximum of 500) and allows various criteria to be specified for each trace. Searches can go forward or backward from the primary (unnamed) marker. If the primary marker has not been set, the search starts at the beginning of the displayed data (\"From\") for a forwards search and starts at the end of the displayed data (\"To\") for a backwards search. \"Mark\" and \"Clear\" are used to modify the normal time vertical markings such that they can be used to indicate all the times that a specific pattern search condition is true (e.g., every upclock of a specific signal)"))
        return;

    for (t = traces.first; t; t = t->next)
    {
        if (TRACE_IS_SEPARATOR (t->config) || !TRACE_IS_SELECTED (t->config) || !t->config->name)
        {
            continue;
        }

        /* at least one good trace, so do it */
        tracesearchbox ("Waveform Display Search", menu_tracesearchbox_callback);
        return;
    }

    wave_log (TRUE, "You must select at least one non-separator, non-comment trace.\n");
}

gboolean on_top_level_delete_event (GtkWidget * widget, GdkEvent * event, gpointer user_data)
{
    gtk_main_quit ();
    return TRUE;
}

/* on_handlebox_toggle_clicked: handle hiding/showing toolbars which the
	toolbar toggle buttons are pressed */
void on_handlebox_toggle_clicked (GtkButton * button, gpointer user_data)
{
    char *handle_box_name = (char *) user_data;
    GtkWidget *main_window = widget_top_parent (GTK_WIDGET (button));
    GtkWidget *handlebox = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (main_window),
      handle_box_name);
    gboolean active = GTK_TOGGLE_BUTTON (button)->active;

    if (active)
        gtk_widget_hide (handlebox);
    else
        gtk_widget_show (handlebox);
}

void on_filter_dialog_ok (GtkButton * button, gpointer user_data)
{
}

void on_filter_dialog_cancel (GtkButton * button, gpointer user_data)
{
}

/* ------8<-----CUT HERE-----8<------- */
