/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 1999-2003 Tony Bybell
    Copyright (C) 2000-2003 Department of Computer Science
    The University of Manchester, Oxford Road, Manchester, M13 9PL, UK

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
    
    `log.h'
    Log window control

    $Id: log.h,v 1.11 2003/07/16 11:24:04 bardslea Exp $
*/

#ifndef WAVE_LOG_H
#define WAVE_LOG_H

#include "error.h"
#include <gtk/gtk.h>

typedef enum 
{
    WAVE_LOG_NEVER, WAVE_LOG_ERROR, WAVE_LOG_ALWAYS
}
WaveLogLevel;

extern WaveLogLevel wave_log_level;

/* wave_log_context : print an error context header for a subsequent wave_log call */
extern void wave_log_context (WaveErrorContext *context);

/* wave_log : error logging to window or stderr */
extern void G_GNUC_PRINTF (2, 3) wave_log (gboolean is_error, const char *fmt, ...);

/* wave_init_log_window : create the log window (if necessary) and raise/show it
	iff `show' is TRUE */
extern void wave_init_log_window (gboolean show);

/* wave_exit : exit GTKWave on a fatal error giving a handy exit-delaying
	dialogue box if we're graphical yet */
extern void wave_exit (int return_val);

/* Callbacks */
extern void on_open_log_activate (GtkMenuItem * menuitem, gpointer user_data);
extern gboolean on_log_window_delete_event (GtkWidget * widget, GdkEvent * event, gpointer user_data);
extern void on_close_log_clicked (GtkButton * button, gpointer user_data);
extern gboolean on_log_window_destroyed (GtkWidget * widget, GdkEvent * event, gpointer user_data);
extern void on_exit_gtkwave_exit_clicked (GtkButton *button, gpointer user_data);

#endif
