/*
	The GTKWave electronic waveform viewer
	Copyright (C) 2000-2003 Department of Computer Science
	The University of Manchester, Oxford Road, Manchester, M13 9PL, UK
	Copyright (C) 1999-2003 Tony Bybell

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

	`marker.c'
	Markers handling (and GUI elements)

	$Id: marker.c,v 1.5 2003/11/11 13:24:50 bardslea Exp $
*/

#include "currenttime.h"
#include "gui_if.h"
#include "support.h"
#include "help.h"
#include "marker.h"
#include "tracepane.h"

TimeType named_markers[26];

static GtkWidget *marker_dialog = NULL;

/*
    Add action on click of marker row?  Every row associated with its
    marker number (=letter-'A')...
*/

static int marker_sort_col, marker_sort_dir;
static TimeType shadow_markers[26];

/* wave_init_marker_dialog : make (or just raise) the marker editing dialog */
void wave_init_marker_dialog (void)
{
    GtkCList *clist;
    char marker[2], posn[49];
    char *ary[2];
    int i, row;

    if (marker_dialog)
    {
        /* Gtk 1.2 has no other way to do this.  :^( */
        gdk_window_raise (marker_dialog->window);
        return;
    }
    marker_dialog = create_marker_dialog ();
    clist = GTK_CLIST (lookup_widget (marker_dialog, "marker_list"));

    ary[0] = marker;
    ary[1] = posn;
    marker[1] = '\0';
    for (i = 0; i < 26; i++)
    {
        marker[0] = 'A' + i;
        if (named_markers[i] == -1)
        {
            sprintf (posn, "<None>");
        } else
        {
            wave_format_time (posn, named_markers[i], wave_config->time_dimension);
        }
        shadow_markers[i] = named_markers[i];
        row = gtk_clist_append (clist, ary);
        gtk_clist_set_row_data (clist, row, GINT_TO_POINTER (i));
    }

    gtk_clist_set_sort_column (clist, 0);
    gtk_clist_set_sort_type (clist, GTK_SORT_ASCENDING);
    gtk_clist_set_auto_sort (clist, TRUE);
    marker_sort_col = 0;
    marker_sort_dir = 0;
    gtk_widget_show (marker_dialog);
}

static void markers_sort (int col)
{
    GtkCList *clist = GTK_CLIST (lookup_widget (marker_dialog, "marker_list"));

    if (marker_sort_col == col)
    {
        marker_sort_dir = !marker_sort_dir;
        gtk_clist_set_sort_type (clist, marker_sort_dir ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
    } else
    {
        gtk_clist_set_sort_column (clist, col);
        gtk_clist_set_sort_type (clist, GTK_SORT_ASCENDING);
        marker_sort_col = col;
        marker_sort_dir = 0;
    }
}

gboolean markers_sort_by_name (GtkWidget * widget, GdkEventButton * event, gpointer user_data)
{
    markers_sort (0);
    return TRUE;
}

gboolean markers_sort_by_position (GtkWidget * widget, GdkEventButton * event, gpointer user_data)
{
    markers_sort (1);
    return TRUE;
}

gboolean on_marker_dialog_delete_event (GtkWidget * widget, GdkEvent * event, gpointer user_data)
{
    gtk_widget_destroy (marker_dialog);
    marker_dialog = NULL;
    return TRUE;
}

void on_marker_list_select_row (GtkCList * clist, gint row, gint column, GdkEvent * event, gpointer user_data)
{
    gint marker_index;
	char *clist_val;
	GtkWidget *entry = lookup_widget (GTK_WIDGET (clist), "marker_entry");

    marker_index = GPOINTER_TO_INT (gtk_clist_get_row_data (clist, row));

	gtk_object_set_data (GTK_OBJECT (widget_top_parent (GTK_WIDGET (clist))),
		"selected_marker_index", GINT_TO_POINTER (marker_index));
	gtk_clist_get_text (clist, marker_index, 1, &clist_val);
    gtk_entry_set_text (GTK_ENTRY (entry), (shadow_markers[marker_index] >= 0 ? clist_val : "<None>"));
	gtk_widget_set_sensitive (lookup_widget (GTK_WIDGET (clist), "marker_delete"), TRUE);
}

#define WAVE_MAX_TIME_LENGTH (40)

void on_marker_entry_activate (GtkEditable *editable, gpointer user_data)
{
	GtkWidget *clist = lookup_widget (GTK_WIDGET (editable), "marker_list");
	TimeType time = wave_parse_time (gtk_entry_get_text (GTK_ENTRY (editable)), wave_config->time_dimension);
	int marker_index = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (
		widget_top_parent (GTK_WIDGET (editable))), "selected_marker_index"));

fprintf (stderr, "%lld %lld %lld\n", time, tims.first, tims.last);

	/* Fail silently if the time is not valid */
	if (time >= tims.first && time <= tims.last)
	{
		char time_str[WAVE_MAX_TIME_LENGTH];

        shadow_markers[marker_index] = time;
        wave_format_time (time_str, time, wave_config->time_dimension);
        gtk_clist_set_text (GTK_CLIST (clist), marker_index, 1, time_str);
	} else {
        shadow_markers[marker_index] = -1;
        gtk_clist_set_text (GTK_CLIST (clist), marker_index, 1, "<None>");
	}
}

void on_marker_delete_clicked (GtkButton *button, gpointer user_data)
{
	int marker_index = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (
		widget_top_parent (GTK_WIDGET (button))), "selected_marker_index"));
	TimeType marker_time = shadow_markers[marker_index];

	shadow_markers[marker_index] = -1;
    gtk_clist_set_text (GTK_CLIST (lookup_widget (GTK_WIDGET (button), "marker_list")), marker_index, 1, "<None>");
	wave_redraw (wave_window);
}

void on_marker_list_unselect_row (GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer user_data)
{
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (GTK_WIDGET (clist), "marker_entry")), "");
	gtk_widget_set_sensitive (lookup_widget (GTK_WIDGET (clist), "marker_delete"), FALSE);
}

void on_markers_close_clicked (GtkButton * button, gpointer user_data)
{
    gtk_widget_destroy (marker_dialog);
    marker_dialog = NULL;
}

void on_markers_accept_clicked (GtkButton * button, gpointer user_data)
{
    int i;

    /* use the cancel behaviour first */
    on_markers_close_clicked (NULL, NULL);

    /* now copy the marker definitions and reconfig */
    for (i = 0; i < 26; i++)
    {
        named_markers[i] = shadow_markers[i];
    }
	wave_redraw (wave_window);
}

void on_markers_menu_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    /* Does nothing for now */
}

void on_markers_edit_markers_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Show-Change Marker Data", "displays the times for all 26 named markers"))
        return;

    wave_init_marker_dialog ();
}

void on_markers_place_marker_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Drop Named Marker", "drops a named marker where the current primary (unnamed) marker is placed. A maximum of 26 named markers are allowed and the times for all must be different"))
        return;

    if (tims.marker != -1)
    {
        int i;

        for (i = 0; i < 26; i++)
        {                       /* only one per slot */
            if (named_markers[i] == tims.marker)
                return;
        }

        for (i = 0; i < 26; i++)
        {                       /* first unused */
            if (named_markers[i] == -1)
            {
                named_markers[i] = tims.marker;
				wave_redraw (wave_window);
                return;
            }
        }
    }
}

void on_markers_remove_marker_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    if (wave_help ("Collect Named Marker", "collects a named marker where the current primary (unnamed) marker is placed if there is a named marker at its position"))
        return;

    if (tims.marker != -1)
    {
        int i;

        for (i = 0; i < 26; i++)
        {
            if (named_markers[i] == tims.marker)
            {
                named_markers[i] = -1;
				wave_redraw (wave_window);
                return;
            }
        }
    }
}

void on_markers_collect_all_named_markers_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
{
    int i, dirty = 0;

    if (wave_help ("Collect All Named Markers", "collects any and all named markers which have been dropped"))
        return;

    for (i = 0; i < 26; i++)
    {
        if (named_markers[i] != -1)
        {
            named_markers[i] = -1;
            dirty = 1;
        }
    }

    if (dirty)
		wave_redraw (wave_window);
}

void on_marker_entry_changed (GtkEditable *editable, gpointer user_data)
{
}

