/*
	The GTKWave electronic waveform viewer
	Copyright (C) 2000-2003 Department of Computer Science
	The University of Manchester, Oxford Road, Manchester, M13 9PL, UK
	Copyright (C) 1999-2003 Tony Bybell

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

	`navigate.h'
	Navigation GUI elements and functions

	$Id: navigate.h,v 1.6 2004/04/25 19:23:42 bardslea Exp $
*/

#ifndef WAVE_NAVIGATE_H
#define WAVE_NAVIGATE_H

#include <gtk/gtk.h>
#include "config/global.h"
#include "view.h"

/* wave_calculate_zoom_for_span : calculate the zoom factor to show the span of
	time from `from' to `to'. */
extern double wave_calculate_zoom_for_span (TimeType from, TimeType to);

/* wave_zoom : zoom the display to zoom level wave_flags.zoom_base^z0, adjust this zoom factor
	to make it sensible */
extern void wave_zoom (gdouble z0, WaveWindow *window);

/* wave_set_{start,end}_time : set start/end time entry text to the given time */
extern void wave_set_start_time (TimeType start_time);
extern void wave_set_end_time (TimeType end_time);

/* wave_set_step_size : set the step size to the given value/dimension */
extern void wave_set_step_size (TimeType time, char time_dimension, WaveWindow *window);

/* GUI */
extern void on_time_menu_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_time_move_to_time_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_time_zoom_zoom_in_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_zoom_out_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_time_zoom_best_fit_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_time_zoom_start_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_time_zoom_end_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_time_zoom_undo_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_time_zoom_redo_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_time_step_step_left_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_time_step_step_right_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_time_page_page_left_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_time_page_page_right_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_page_left_clicked (GtkButton * button, gpointer user_data);
extern void on_page_right_clicked (GtkButton * button, gpointer user_data);
extern void on_from_entry_changed (GtkEditable * editable, gpointer user_data);
extern void on_to_entry_changed (GtkEditable * editable, gpointer user_data);
extern void on_step_size_entry_changed (GtkEditable * editable, gpointer user_data);
extern void on_step_left_clicked (GtkButton * button, gpointer user_data);
extern void on_step_right_clicked (GtkButton * button, gpointer user_data);
extern void service_dragzoom (TimeType time1, TimeType time2);
extern void fix_wavehadj (WaveWindow *window);
extern void clicked_zoom_out (GtkButton * button, gpointer user_data);
extern void clicked_zoom_in (GtkButton * button, gpointer user_data);
extern void clicked_zoom_fit (GtkButton * button, gpointer user_data);
extern void clicked_zoom_start (GtkButton * button, gpointer user_data);
extern void clicked_zoom_end (GtkButton * button, gpointer user_data);
extern void clicked_zoom_undo (GtkButton * button, gpointer user_data);
extern void clicked_zoom_redo (GtkButton * button, gpointer user_data);
extern void on_move_dialog_cancel_clicked (GtkButton *button, gpointer user_data);
extern void on_move_dialog_ok_clicked (GtkButton *button, gpointer user_data);
extern gboolean on_move_dialog_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
extern void on_move_entry_activate (GtkEditable *editable, gpointer user_data);

#endif
