/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 1999-2003 Tony Bybell
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
    
    `polling.c'
    Periodic file polling

    $Id: polling.c,v 1.16 2003/11/09 23:36:22 bardslea Exp $
*/

#include "polling.h"
#include "main.h"
#include "log.h"
#include "rc.h"
#include "support.h"

/* wave_reread_end_of_file : update traces by re-reading the end of the named file
	assumes that the reader is capable of this */
void wave_reread_end_of_file (WaveReader *reader)
{
	wave_reader_seek_end (reader);
}

/* poll_file : callback to poll a file for new size */
static int poll_file (WaveFilePollingInfo *polling_info)
{
    size_t current_size = get_file_size (polling_info->filename);

    if (current_size > polling_info->prev_size)
    {
        /* Update format data! */
		wave_reread_end_of_file (polling_info->reader);
        polling_info->prev_size = current_size;
    }
    return TRUE;
}

/* wave_install_file_polling : make the named file polled for new additions. */
WaveFilePollingInfo *wave_install_file_polling (WaveReader *reader,
	int polling_time)
{
    WaveFilePollingInfo *polling_info = g_new (WaveFilePollingInfo, 1);

    polling_info->filename = g_strdup (reader->filename);
    polling_info->prev_size = get_file_size (reader->filename);
    polling_info->reader = reader;

	wave_log (FALSE, "Installing polling on file `%s', every %d s\n",
		reader->filename, polling_time);

    polling_info->timeout_ref = gtk_timeout_add (polling_time * 1000,
		(GtkFunction) poll_file, (void *) polling_info);

	return polling_info;
}

/* wave_unpoll_file : unpoll a file and destroy the polling structure */
void wave_unpoll_file (WaveFilePollingInfo *polling_info)
{
	if (! polling_info)
		return;

	g_free (polling_info->filename);
	gtk_timeout_remove (polling_info->timeout_ref);
	g_free (polling_info);
}

void on_re_read_end_of_file_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	wave_reread_end_of_file (wave_reader);
}

void on_periodically_reread_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	gboolean active = GTK_CHECK_MENU_ITEM (menuitem)->active;

	if (wave_preferences_sensitive)
	{
		wave_unpoll_file (wave_polling);
		wave_polling = NULL;

		if (active)
			wave_polling = wave_install_file_polling (wave_reader, 1);
	}
}

/* wave_set_periodically_reread_active : set/clear `periodically reread file' toggle
	button in File menu */
void wave_set_periodically_reread_active (gboolean active)
{
	GtkWidget *poll_menuitem = lookup_widget (wave_window->window, "periodically_reread");

	wave_preferences_sensitive = FALSE;
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (poll_menuitem), active);
	wave_preferences_sensitive = TRUE;
}
