/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 1999-2003 Tony Bybell
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
    
    `polling.h'
    Periodic file polling

    $Id: polling.h,v 1.10 2003/07/11 15:21:15 bardslea Exp $
*/

#ifndef WAVE_POLLING_H
#define WAVE_POLLING_H

#include "reader.h"

/* WaveFilePollingInfo : information used to poll files for changes of size */
typedef struct
{
    char *filename;
    size_t prev_size;
    WaveReader *reader;
	guint timeout_ref; 	/* Internal GTK+ reference to the set timeout */
}
WaveFilePollingInfo;

/* wave_reread_end_of_file : update traces by re-reading the end of the named file
	assumes that the reader is capable of this */
extern void wave_reread_end_of_file (WaveReader *reader);

/* install_file_polling : make the named file polled for new additions. */
extern WaveFilePollingInfo *wave_install_file_polling (WaveReader *reader,
	int polling_time);

/* wave_unpoll_file : unpoll a file and destroy the polling structure */
extern void wave_unpoll_file (WaveFilePollingInfo *polling_info);

/* Callbacks */
extern void on_re_read_end_of_file_activate (GtkMenuItem *menuitem, gpointer user_data);
extern void on_periodically_reread_activate (GtkMenuItem *menuitem, gpointer user_data);

/* wave_set_periodically_reread_active : set/clear `periodically reread file' toggle
	button in File menu */
extern void wave_set_periodically_reread_active (gboolean active);

#endif
