/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 2003 Department of Computer Science
    The University of Manchester, Oxford Road, Manchester, M13 9PL, UK
    Copyright (C) 1999-2003 Tony Bybell
    Copyright (C) 2000 Udi Finkelstein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    `print.c'
    Printing

    $Id: print.h,v 1.11 2003/05/05 20:27:14 bardslea Exp $
*/

#ifndef WAVE_PRINT_H
#define WAVE_PRINT_H

#include <gtk/gtk.h>
#include "draw.h"

struct WavePrinter_;

/* WavePrintingContext : contains all printing information for a particular printing
	operation instance */
typedef struct
{
	WaveDrawingContext *drawing_context;
    struct WavePrinter_ *printer;
    double page_width;         /* Legal page width (mm) */
    double page_height;        /* Legal page height (mm) */
    double left_margin;        /* Left Margin (inch) */
    double right_margin;       /* Right Margin (inch) */
    double bottom_margin;      /* Bottom Margin (inch) */
    double top_margin;         /* Top Margin (inch) */
    double scale;				/* mm per pixel */
    gboolean rotate;            /* FALSE: portrait, TRUE: landscape */
    double gray;
    int minimum_x, minimum_y, maximum_x, maximum_y; /* These will be initialized by a routine */
}
WavePrintingContext;

/* WavePrinter : describes a print format functions/options separate from the drawing functions 
	defined in draw.c */
typedef struct WavePrinter_
{
	char *format_name;
	void (*print) (WavePrintingContext *context, FILE *file, double px, double py, gboolean rotate);
    void (*header) (WavePrintingContext *context);
    void (*trailer) (WavePrintingContext *context);
    void (*traces_init) (WavePrintingContext *context);
}
WavePrinter;

/* wave_print_image : print using a fully set up print context */
extern void wave_print_image (WavePrintingContext *context);

/* wave_init_print_dialog : create/show print dialogue box */
extern void wave_init_print_dialog (void);

/* Callbacks */
extern void on_print_file_do_select_clicked (GtkButton * button, gpointer user_data);
extern void on_print_cancel_clicked (GtkButton * button, gpointer user_data);
extern void on_print_ok_clicked (GtkButton * button, gpointer user_data);
extern void on_print_file_selector_ok_clicked (GtkButton * button, gpointer user_data);
extern void on_print_file_selector_cancel_clicked (GtkButton * button, gpointer user_data);
extern void on_file_print_activate (GtkMenuItem * menuitem, gpointer user_data);

#endif
