/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 2003 Department of Computer Science
    The University of Manchester, Oxford Road, Manchester, M13 9PL, UK
    Copyright (C) 2000 Udi Finkelstein
    Copyright (C) 1999-2003 Tony Bybell

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    `printps.c'
    Postscript printing stuff

    $Id: printps.c,v 1.6 2003/11/13 11:05:38 bardslea Exp $
*/

#include <errno.h>
#include <stdio.h>
#include <time.h>
#include "printps.h"
#include "draw.h"
#include "log.h"
#include "rc.h"
#include "currenttime.h"
#include "signalpane.h"

static void ps_print (WavePrintingContext *context, FILE *file, double px, double py, gboolean rotate)
{
    context->drawing_context = wave_new_ps_drawing_context (file, wave_window);
    context->page_width = px;        /* Legal page width */
    context->page_height = py;       /* Legal page height */
    context->left_margin = 1;        /* Left Margin (inch) */
    context->right_margin = 1;       /* Right Margin (inch) */
    context->bottom_margin = 1;      /* Bottom Margin (inch) */
    context->top_margin = 1;         /* Top Margin (inch) */
    context->rotate = rotate;

    wave_print_image (context);
}

static void ps_traces_init (WavePrintingContext * context)
{
    FILE *file = context->drawing_context->data.ps.file;
    fprintf (file, "%d 0 translate 0.5 setlinewidth stroke\n", wave_window->signals.pixmap_width);
}

static void ps_prolog (WavePrintingContext *context)
{
    FILE *file = context->drawing_context->data.ps.file;

    fprintf (file,
      "/r { %% stack: x1 y1 x2 y2\n"
      "\tnewpath\n"
      "\t2 copy moveto %% x2 y2\n"
      "\t3 index 1 index lineto %% x1 y2\n"
      "\t3 index 3 index lineto %% x1 y1\n"
      "\t1 index 3 index lineto %% x2 y1\n"
      "\tpop pop pop pop closepath} def\n"
      "/fr { r fill } def\n"
	  "/s { /str exch def /y exch def /x exch def gsave\n"
		"x y translate\n"
		"0 0 moveto\n"
		"1 -1 scale\n"
		"str show stroke\n"
		"grestore } def\n"
      "/l { %% stack: x1 y1 x2 y2\n" "\tnewpath moveto lineto closepath stroke} def\n"
      "/rgb { setrgbcolor } def\n");
}

static void ps_eps_header_misc (FILE *file)
{
	time_t now = time (NULL);

    fprintf (file, "%%Title: need_title\n%%Creator: %s\n"
    	"%%CreationDate: %s%%For: %s\n",
    	WAVE_TITLE, ctime (&now), g_get_user_name ());
}

static void ps_header (WavePrintingContext * context)
{
    FILE *file = context->drawing_context->data.ps.file;

    fprintf (file, "%%!PS-Adobe-2.0\n");

    ps_eps_header_misc (file);
    ps_prolog (context);

    fprintf (file, "/Courier findfont 10 scalefont setfont\n");
    fprintf (file, "2 setlinecap gsave 1 1 scale 0.5 setlinewidth stroke\n");
    fprintf (file, "72 25.4 div dup scale %% Set scale to mm\n");
    fprintf (file, "%f -%f scale\n", context->scale, context->scale);
    fprintf (file, "0 -%f translate\n", context->page_height / context->scale);
}

static void eps_header (WavePrintingContext * context)
{
    FILE *file = context->drawing_context->data.ps.file;

    fprintf (file, "%%!PS-Adobe-2.0 EPSF-2.0\n");

    ps_eps_header_misc (file);
    ps_prolog (context);

    fprintf (file, "/Courier findfont 10 scalefont setfont\n");
    fprintf (file, "2 setlinecap gsave 1 1 scale 0.5 setlinewidth stroke\n");
    fprintf (file, "72 25.4 div dup scale %% Set scale to mm\n");
    fprintf (file, "%f -%f scale\n", context->scale, context->scale);
    fprintf (file, "0 -%f translate\n", context->page_height / context->scale);
}

static void ps_trailer (WavePrintingContext * context)
{
    FILE *file = context->drawing_context->data.ps.file;

    fprintf (file, "grestore showpage\n");
}

WavePrinter wave_printer_ps =
{
	"PostScript (Level 2)",
	ps_print, ps_header, ps_trailer, ps_traces_init
};

/* FIXME */
WavePrinter wave_printer_eps =
{
	"Encapsulated PostScript",
	ps_print, eps_header, ps_trailer, ps_traces_init
};
