/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 2004 Andrew Bardsley
    Copyright (C) 1999-2003 Tony Bybell
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
    
    `rc.h'
    Configuration file handling and preferences dialogue

    $Id: rc.h,v 1.28 2004/04/25 19:23:42 bardslea Exp $
*/

#ifndef WAVE_RC_H
#define WAVE_RC_H

#include <stdio.h>
#include <gtk/gtk.h>
#include "color.h"
#include "format/format.h"

#define WAVE_RADIX_COUNT (4)

typedef struct
{
	FormatRadixData *radices[WAVE_RADIX_COUNT];
}
WaveRadixStyle;

/* wave_radix_style : styles of display for the 4 supported radices */
extern WaveRadixStyle *wave_radix_style;

/* Central place to put flags, consider making this part
	of wave_config and the config suite in general */
/* Oh yeah, get rid of some of these too */
typedef struct
{
    gboolean autocoal_flag;
    gboolean autocoal_rev_flag;
    gboolean autoname_flag; /* to dispense with having to hit ok on bundles (let gtkwave name them) */
    gboolean center_zooms_flag;
    gboolean const_mark_update_flag; /* true if you want hold downs to const update at expense of speed */
    gboolean do_initial_zoom_fit; /* when set forces an initial zoom fit */
    gboolean enable_ghost_marker; /* says whether to draw ghost markers */
    gboolean enable_horiz_grid; /* says whether to draw horiz rulers on traces */
    gboolean enable_vert_grid; /* says whether to draw vert rulers on traces */
    gboolean pow10snap_flag; /* forces divisible by 10 timescale no matter what the zoom is */
    gboolean roundcap_vectors_flag; /* for draw_vector_ent_trace */
    gboolean search_hier_group_flag;
    gboolean show_base_flag; /* if false, omits leading base info */
    gboolean show_grid_flag;
    gboolean values_below_flag; /* traces with heights > 1 show have their values shown on the
    	second line of the trace in the signals pane rather than after the = */
    gboolean sig_just_flag; /* when enabled, signals are left, rather than right flushed */

    int vector_padding;		/* padding between vector values and lines in display */

    int window_width, window_height, signal_width; /* Prefered geometry values */

    gdouble zoom_base; /* zoom in/out multiple */

    /* Window element visibility flags */
    gboolean show_menubar, show_operations, show_navigation, show_times;
}
WaveFlags;

extern WaveFlags wave_flags;

/* Callbacks for preferences should be reactive when this flag is true */
extern gboolean wave_preferences_sensitive;

typedef struct
{
    char *name;
    int (*parse) (char *, int);
    void (*write) (FILE *, char *, int);
    int offset;
}
WaveRcEntry;

/* wave_init_flags : initialise flags to preferred values, call this
	before parsing rc file */
extern void wave_init_flags (void);

extern void wave_read_rc_file (void);
extern void wave_parse_reader_rc_entries (int reader_entryc, WaveRcEntry * reader_entryv);

/* wave_parse_boolean_rc_entry : parse a boolean rc file option.  Anything
	starting with [yY] or a number that isn't 0 is true */
extern gboolean wave_parse_boolean_rc_entry (const char *str);

/* wave_populate_preferences_dialog : fill the preferences dialogue from the
	settings in tool at the moment gathered from:
		wave_flags, wave_log_level ... */
extern void wave_populate_preferences_dialog (GtkWidget *preferences);

/* wave_find_radix_data : find radix info for the given radix (2, 8, 16, 10).  Returns NULL on failure */
extern FormatRadixData *wave_find_radix_data (unsigned radix);

/* Callbacks */
extern void on_preferences_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_save_preferences_activate (GtkMenuItem * menuitem, gpointer user_data);
extern void on_autocoal_toggled (GtkToggleButton * togglebutton, gpointer user_data);
extern void on_autocoal_rev_toggled (GtkToggleButton * togglebutton, gpointer user_data);
extern void on_autoname_toggled (GtkToggleButton * togglebutton, gpointer user_data);
extern void on_search_hier_group_toggled (GtkToggleButton * togglebutton, gpointer user_data);
extern void on_show_grid_toggled (GtkToggleButton * togglebutton, gpointer user_data);
extern void on_values_below_toggled (GtkToggleButton * togglebutton, gpointer user_data);
extern void on_show_base_toggled (GtkToggleButton * togglebutton, gpointer user_data);
extern void on_const_mark_update_toggled (GtkToggleButton * togglebutton, gpointer user_data);
extern void on_pow10snap_toggled (GtkToggleButton * togglebutton, gpointer user_data);
extern void on_page_length_changed (GtkEditable * editable, gpointer user_data);
extern void on_zoom_factor_changed (GtkEditable * editable, gpointer user_data);
extern void on_log_never_clicked (GtkButton * button, gpointer user_data);
extern void on_log_error_clicked (GtkButton * button, gpointer user_data);
extern void on_log_always_clicked (GtkButton * button, gpointer user_data);
extern void on_roundcap_vectors_toggled (GtkToggleButton * togglebutton, gpointer user_data);
extern void on_sig_just_left_clicked (GtkButton * button, gpointer user_data);
extern void on_sig_just_right_clicked (GtkButton * button, gpointer user_data);
extern void on_preferences_apply_clicked (GtkButton * button, gpointer user_data);
extern void on_preferences_close_clicked (GtkButton * button, gpointer user_data);
extern gboolean on_preferences_delete_event (GtkWidget * widget, GdkEvent * event, gpointer user_data);
extern void on_radix_list_unselect_row (GtkCList *clist, gint row, gint column, GdkEvent *event,
	gpointer user_data);
extern void on_radix_list_select_row (GtkCList *clist, gint row, gint column, GdkEvent *event,
	gpointer user_data);
extern void on_radix_prefix_entry_activate (GtkEditable *editable, gpointer user_data);
extern void on_radix_suffix_entry_activate (GtkEditable *editable, gpointer user_data);
extern void on_radix_show_leading_zeroes_toggled (GtkToggleButton *togglebutton, gpointer user_data);
extern void on_radix_separator_spacing_changed (GtkEditable *editable, gpointer user_data);
extern void on_radix_copy_style_button_clicked (GtkButton *button, gpointer user_data);

#endif
