/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 2004 Andrew Bardsley
    Copyright (C) 1999-2003 Tony Bybell
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
    
    `str.c'
    String handling helper functions

    $Id: str.c,v 1.9 2004/04/25 19:23:42 bardslea Exp $
*/

#include <glib.h>
#include <string.h>
#include <ctype.h>

/* wave_get_filename_extension : return the first extension off a filename.
	This will either be NULL (no extension) or a pointer into the
	string. *Not* a newly allocated string. */
char *wave_get_filename_extension (const char *filename)
{
    char *dot_position = strrchr (filename, '.');

    return (dot_position ? dot_position + 1 : NULL);
}

/* wave_skip_ws : returns the tail of a string after the leading whitespace */
char *wave_skip_ws (char *str)
{
	while (isspace (*str)) str ++;

	return str;
}

/* wave_substitute_char : substitute all occurences of one char for another in
	the given string, modifies the string and returns a pointer to the start of
	the string */
char *wave_substitute_char (char *str, char from, char to)
{
	char *string_ptr = strchr (str, from);

	while (string_ptr)
	{
		*string_ptr = to;
		string_ptr = strchr (str, from);
	}

	return str;
}

/* wave_strcpy : do an strcpy of `src' into `dest' and return a pointer
	to the new NUL termination on `src' */
char *wave_strcpy (char *dest, const char *src)
{
	while (*src)
	{
		*dest = *src;

		src ++;
		dest ++;
	}
	*dest = '\0';

	return dest;
}

/* wave_end_of_string : returns a pointer to the NUL termination on the given string */
char *wave_end_of_string (char *str)
{
	while (*str) str++;

	return str;
}
