/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 1999-2003 Tony Bybell
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
    
    `support.h'
    GTK+ Support Functions

    $Id: support.h,v 1.14 2003/05/05 20:27:15 bardslea Exp $
*/

#ifndef WAVE_SUPPORT_H
#define WAVE_SUPPORT_H

#include "../config.h"

#include <gtk/gtk.h>

/* widget_top_parent : get the top parent (the parent that has no parent) of the
	given widget */
extern GtkWidget *widget_top_parent (GtkWidget *widget);

/* lookup_widget : lookup a widget by finding a data entry in the top
	parent of the given widget */
extern GtkWidget *lookup_widget (GtkWidget * widget, const gchar * widget_name);

extern GtkWidget *create_pixmap (GtkWidget * widget, const gchar * filename);

/* wave_register_pixmaps : add the pixmap directory to the pixmap path and register
	builtin pixmaps */
extern void wave_register_pixmaps (GtkWidget * widget);

/* wave_get_option_menu_active_data : get a named data element from the active element of
	the given option menu, returns NULL on any errors */
extern gpointer wave_get_option_menu_active_data (GtkOptionMenu * option_menu, const char *data_element_name);

/* wave_get_option_menu_index : get the index of the active element in the given option menu */
extern guint wave_get_option_menu_index (GtkOptionMenu * option_menu);

/* wave_option_menu_set_menu_from_string_array : remove the current menu from
	an option menu and replace with a menu composed of the strings provided.
	If `data_entry_name' is not NULL then set data for each entry to be either
	the index of the entry (if `dataNindex' is FALSE) or to the element in
	the array `data' matching the index of this entry (if `dataNindex' is TRUE).
	If `callback' is not NULL then set an activate callback, passed the same data as set
	for each enrty, for each entry. */
extern void wave_option_menu_set_menu_from_string_array (GtkOptionMenu *option_menu,
	char **strings, void **data, char *data_entry_name, gboolean dataNindex,
	void (* callback) (GtkMenuItem *, void *));

/* wave_set_sensitivity_of_named_widgets : set each of the named widgets'
	sensitivities.  `widget_names' entries are found by using lookup_widget */
extern void wave_set_sensitivity_of_named_widgets (GtkWidget *reference_widget,
	char **widget_names, gboolean sensitivity);

/* wave_show_dialog : raise/show/otherwise draw the user's attention to the
	dialogue box `dialog' (which must actually be a valid widget) */
extern void wave_show_dialog (GtkWidget *dialog);

#endif
