/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 1999-2003 Tony Bybell
    Copyright (C) 2000-2003 Department of Computer Science
    The University of Manchester, Oxford Road, Manchester, M13 9PL, UK

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    `utils.c'
    Utility functions

    $Id: utils.c,v 1.35 2003/07/20 20:30:22 bardslea Exp $
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "utils.h"
#include "bsearch.h"
#include "tree.h"
#include "log.h"

static Symbol **sym = NULL;
Symbol **wave_symbols = NULL;
gboolean wave_symbols_are_sorted = FALSE;

int wave_num_symbols = 0;
int longestname = 0;

Symbol *firstnode = NULL;       /* 1st sym in loaded file */

#define SYMPRIME 65519

/*
    Generic hash function for symbol names...
*/
int hashcache;

int hash (char *s)
{
    char *p;
    char ch;
    unsigned int h = 0, h2 = 0, pos = 0, g;

    for (p = s; *p; p++)
    {
        ch = *p;
        h2 <<= 3;
        h2 -= ((unsigned int) ch + (pos++)); /* this handles stranded vectors quite well.. */

        h = (h << 4) + ch;
        g = h & 0xf0000000;
        if (g)
        {
            h ^= (g >> 24) ^ g;
        }
    }

    h ^= h2;                    /* combine the two hashes */
    hashcache = h % SYMPRIME;
    return hashcache;
}

/*
    add symbol to table.  no duplicate checking; assume "correct."
*/
Symbol *symadd (char *name, int hv)
{
    Symbol *s = g_new (Symbol, 1);

    s->name = g_strdup (name);
    s->next = sym[hv];
	s->next_in_file = NULL;
	s->vec_root = s->vec_chain = NULL;
	s->selected = FALSE;
	s->n = NULL;
    sym[hv] = s;

    return s;
}

/*
    find a slot already in the table...
*/
Symbol *symfind (char *s)
{
    if (!wave_symbols_are_sorted)
    {
        int hv = hash (s);
        Symbol *temp = sym[hv];

        if (!temp)
            return NULL;        /* no hash entry, add here wanted to add */

        for (; temp; temp = temp->next)
        {
            if (!strcmp (temp->name, s))
            {
                return temp;    /* in table already */
            }
            /* if (!temp->next) break; */
        }

        return NULL;            /* not found, add here if you want to add */
    } else
    {                           /* no sense hashing if the wave_symbols table is built */
        WAVE_DEBUG (wave_log (FALSE, "WAVE_DEBUG: BSEARCH: %s\n", s));
        return bsearch_symbols (s);
    }
}

/* symnew : make a new symbol table of size SYMPRIME */
void symnew (void)
{
	int i;

    sym = g_new (Symbol *, SYMPRIME);

    for (i = 0; i < SYMPRIME; i ++)
    	sym[i] = NULL;
}

/* symdump : dump a symbol table to a stream, `symbol_count' can be used
	to specify the table length or -1 for a hash accessible (fixed size!)
	table. */
void symdump (FILE * stream, Symbol ** symbols, int symbol_count)
{
    Symbol *symbol;
    int i;

    if (symbol_count == -1)
        symbol_count = SYMPRIME;
    for (i = 0; i < symbol_count; i++)
    {
        symbol = symbols[i];

        while (symbol)
        {
            if (symbol->n)
            {
                Node *node = symbol->n;

                fprintf (stream, "%d: `%s'", i, node->nname);

                if (node->ext)
                    fprintf (stream, " msi=%d lsi=%d", node->ext->msi, node->ext->lsi);
            } else
                fprintf (stream, "%d: No node", i);
            putc ('\n', stream);
            symbol = symbol->next;
        }
    }
}

/*
 * Version of strcpy that changes the hierarchy delimiter if an
 * alternate one is specified.
 */
void copy_change_delim (char *dst, const char *src)
{
    strcpy (dst, src);

    if (alt_hier_delimiter)
    {
        while (dst)
        {
            dst = strchr (dst, alt_hier_delimiter);

            if (dst)
                *dst = hier_delimiter;
        }
    }
}
