/*    
    The GTKWave electronic waveform viewer
    Copyright (C) 1999-2003 Tony Bybell
    Copyright (C) 2000-2003 Department of Computer Science
    The University of Manchester, Oxford Road, Manchester, M13 9PL, UK

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
    
    `utils.h'
    Utility functions

    $Id: utils.h,v 1.29 2003/08/05 22:55:05 bardslea Exp $
*/

#ifndef WAVE_UTILS_H
#define WAVE_UTILS_H

#include <stdio.h>
#include "analyzer.h"
#include "currenttime.h"
#include "tree.h"

typedef struct Symbol_
{
    struct Symbol_ *next_in_file;  /* for file node chaining */
    struct Symbol_ *vec_root, *vec_chain;
    struct Symbol_ *next;       /* for hash chain */
    char *name;
    gboolean selected;          /* for the clist object */
    Node *n;
}
Symbol;

/* symnew : make a new symbol table of size SYMPRIME */
extern void symnew (void);
extern Symbol *symfind (char *);
extern Symbol *symadd (char *, int);
extern int hash (char *s);

/* symdump : dump a symbol table to a stream, `symbol_count' can be used
	to specify the table length or -1 for a hash accessible (fixed size!)
	table. */
extern void symdump (FILE * stream, Symbol ** symbols, int symbol_count);

void copy_change_delim (char *dst, const char *src);

int sigcmp (char *, char *);
void quicksort (Symbol **, int, int);
void wave_heapsort (Symbol ** a, int num);

extern Symbol **wave_symbols;
extern int wave_num_symbols, longestname, hashcache;
extern Symbol *firstnode;
extern gboolean wave_symbols_are_sorted;

extern char *makename_chain (Symbol * sym);


#endif
