/*
    The GTKWave electronic waveform viewer
    Copyright (C) 1999-2003 Tony Bybell
    Copyright (C) 2000-2003 Department of Computer Science
    The University of Manchester, Oxford Road, Manchester, M13 9PL, UK

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    `view.c'
    Trace window representing type

    $Id: view.c,v 1.6 2003/11/13 11:05:38 bardslea Exp $
*/

#include "support.h"
#include "view.h"
#include "tracepane.h"
#include "signalpane.h"
#include "gui_if.h"
#include "navigate.h"
#include "fonts.h"
#include "rc.h"

WaveWindow *wave_window = NULL;

/* wave_new_window : make a new wave window structure */
extern WaveWindow *wave_new_window (void)
{
	WaveWindow *ret = g_new (WaveWindow, 1);

	ret->signals.drawing_area = NULL;
	ret->signals.pixmap = NULL;
	ret->signals.window = NULL;
	ret->signals.window_width_dirty = TRUE;
	ret->signals.hslider = NULL;
	ret->signals.name_width = WAVE_MIN_SIGNAL_PANE_WIDTH;
	ret->signals.pixmap_width = WAVE_MIN_SIGNAL_PANE_WIDTH;
	ret->signals.fill_width = 0;
	ret->signals.pixmap_height = 0;
	ret->signals.fill_height = 0;
	ret->signals.shift_click_trace = NULL;

	ret->traces.drawing_area = NULL;
	ret->traces.hslider = NULL;
	ret->traces.vslider = NULL;
	ret->traces.top_visible_trace = NULL;

	ret->traces.scale.time_units_per_frame = 1;
	ret->traces.scale.pixels_per_frame = 1;
	ret->traces.scale.tickstep = 1;
	ret->traces.scale.pixels_per_time_unit = 1.0;
	ret->traces.scale.time_units_per_pixel = 1.0;
	ret->traces.scale.wave_pane_height = 1;
	ret->traces.scale.wave_pane_width = 1;
	ret->traces.scale.time_line_y = 1;

	wave_window = ret;

    ret->window = create_top_level ();

	gtk_widget_set_usize (ret->window, wave_flags.window_width, wave_flags.window_height);

	/* Hide/show menu/button bars */
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (ret->window, "menubar_toggle_button")),
		! wave_flags.show_menubar);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (ret->window, "operations_toggle_button")),
		! wave_flags.show_operations);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (ret->window, "navigation_toggle_button")),
		! wave_flags.show_navigation);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (ret->window, "times_toggle_button")),
		! wave_flags.show_times);
    
    wave_create_trace_pane (ret);
    wave_load_all_fonts (ret); /* must do before create_signal_pane() */
    wave_create_signal_pane (ret);

    gtk_widget_set_usize (ret->signals.window, wave_flags.signal_width, -1);

	wave_set_step_size (wave_config->time_scale, wave_config->time_dimension, ret);

    update_markertime (tims.marker);

    gtk_widget_show (ret->window);

	return ret;
}
